using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure;
using System.Data.Services.Client;
using System.Diagnostics;

namespace Benday.SequenceGenerator.Business
{
    public class SequenceInformationService
    {
        private const string TABLE_NAME = "SequenceInformation";
        public SequenceInformationService()
        {

        }

        private CloudStorageAccount m_account;
        private CloudStorageAccount Account
        {
            get
            {
                if (m_account == null)
                {
                    m_account = CloudStorageAccount.DevelopmentStorageAccount;
                }

                return m_account;
            }
        }

        private CloudTableClient m_client;
        private CloudTableClient Client
        {
            get
            {
                if (m_client == null)
                {
                    m_client = Account.CreateCloudTableClient();
                }

                return m_client;
            }
        }

        private TableServiceContext m_context;
        public TableServiceContext Context
        {
            get
            {
                if (m_context == null)
                {
                    Client.CreateTableIfNotExist(TABLE_NAME);

                    m_context = Client.GetDataServiceContext();                    
                }
                
                return m_context;
            }
        }

        public IList<SequenceInformation> GetList()
        {
            var query = Context.CreateQuery<SequenceInformation>(TABLE_NAME);

            IList<SequenceInformation> results = new List<SequenceInformation>();

            foreach (var item in query)
            {
                results.Add(item);
            }

            return results;
        }

        public SequenceInformation Get(string name)
        {
            try
            {
                var query = Context.CreateQuery<SequenceInformation>(TABLE_NAME);
                return (from temp in query
                        where 
                            temp.Name == name &&
                            temp.PartitionKey == "default"
                        select temp).FirstOrDefault();
            }
            catch (DataServiceQueryException ex)
            {
                Trace.TraceError(
                        String.Format("SequenceInformationService.Get('{0}') got an exception.  -- {1}", name, ex.ToString()));

                return null;
            }
        }

        public SequenceInformation Save(SequenceInformation saveThis)
        {
            var query = Context.CreateQuery<SequenceInformation>(TABLE_NAME);
            
            if (saveThis.RowKey == Guid.Empty.ToString())
            {
                saveThis.RowKey = Guid.NewGuid().ToString();
                saveThis.PartitionKey = "default";
                Context.AddObject(TABLE_NAME, saveThis);
            }
            else
            {
                Context.UpdateObject(saveThis);
            }
            
            Context.SaveChanges();

            return saveThis;
        }
    }
}
