﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure;
using Benday.SequenceGenerator.Business;

namespace Benday.SequenceGenerator.Workers
{
    public class WorkerRole : RoleEntryPoint
    {
        private SequenceQueueManager m_sequenceQueueManagerInstance;
        public SequenceQueueManager SequenceQueueManagerInstance
        {
            get
            {
                if (m_sequenceQueueManagerInstance == null)
                {
                    m_sequenceQueueManagerInstance = new SequenceQueueManager();
                }

                return m_sequenceQueueManagerInstance;
            }
        }

        public override void Run()
        {
            // This is a sample worker implementation. Replace with your logic.
            Trace.WriteLine("Benday.SequenceGenerator.Workers entry point called", "Information");

            while (true)
            {
                Thread.Sleep(1000);
                Trace.WriteLine("Working", "Information");

                try
                {
                    SequenceQueueManagerInstance.ProcessMessages();
                    SequenceQueueManagerInstance.CheckSequenceLengths();    
                }
                catch (Exception ex)
                {
                    Trace.TraceError(String.Format("WorkerRole got an exception: {0}", ex.ToString()));   
                }
            }
        }

        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            DiagnosticMonitor.Start("DiagnosticsConnectionString");

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            RoleEnvironment.Changing += RoleEnvironmentChanging;

            return base.OnStart();
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // If a configuration setting is changing
            if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
            {
                // Set e.Cancel to true to restart this role instance
                e.Cancel = true;
            }
        }
    }
}
